@@ -59,7 +59,9 @@ def map_value(
     return _map_value(value, resourceOrCxt, False, applyTemplates)
 
 
-def _map_value(value: Any, ctx: "RefContext", wantList: bool = False, applyTemplates: bool = True) -> Any:
+def _map_value(
+    value: Any, ctx: "RefContext", wantList: bool = False, applyTemplates: bool = True
+) -> Any:
     from .support import is_template, apply_template
 
     if Ref.is_ref(value):
@@ -138,8 +140,8 @@ class RefContext:
         wantList: Optional[Union[bool, str]] = False,
         resolveExternal: bool = False,
         trace: Optional[int] = None,
-        strict: bool = _defaultStrictness,
-        task: "TaskView" = None,
+        strict: Optional[bool] = None,
+        task: Optional["TaskView"] = None,
     ) -> None:
         self.vars = vars or {}
         # the original context:
@@ -160,14 +162,16 @@ class RefContext:
         self.kw: MappingType[str, Any] = {}
 
     @property
-    def strict(self):
-        if self.task:
+    def strict(self) -> bool:
+        if self._strict is not None:
+            return self._strict
+        elif self.task:
             return not self.task._rendering
         else:
-            return self._strict
+            return _defaultStrictness
 
     @property
-    def environ(self):
+    def environ(self) -> Dict[str, str]:
         if self.task:
             return self.task.environ
         else:
@@ -340,7 +344,7 @@ class Ref:
         self,
         ctx: RefContext,
         wantList: Union[bool, str] = True,
-        strict: bool = _defaultStrictness,
+        strict: Optional[bool] = None,
     ) -> Optional[Union[ResultsList, Result, List[Result], Any]]:
         """
         If wantList=True (default) returns a ResultList of matches
@@ -394,7 +398,8 @@ class Ref:
                     return list(results)
 
     def resolve_one(
-        self, ctx: RefContext, strict: bool = _defaultStrictness
+        self, ctx: RefContext,
+        strict: Optional[bool] = None,
     ) -> Optional[Union[ResultsList, Result, List[Result]]]:
         """
         If no match return None
@@ -484,6 +489,7 @@ def validate_schema_func(arg, ctx):
     assert_form(args, MutableSequence, len(args) == 2)  # type: ignore
     return validate_schema(args[0], args[1])
 
+
 if sys.version_info >= (3, 9):
     MutableSequence_Result = MutableSequence[Result]
 else:
@@ -493,6 +499,7 @@ ResultOrResultList = Union[Result, MutableSequence_Result]
 PairResultOrResultList = Tuple[Any, ResultOrResultList]
 MaybePairOrResultOrResultList = Union[PairResultOrResultList, ResultOrResultList]
 
+
 def _for_each(
     foreach: Union[MappingType[str, Any], str],
     results: Iterable[Tuple[Any, Any]],
@@ -553,7 +560,9 @@ def _for_each(
                 yield valResult
 
     if keyExp:
-        return [Result(CommentedMap( cast(Iterable[PairResultOrResultList], make_items()) ))]
+        return [
+            Result(CommentedMap(cast(Iterable[PairResultOrResultList], make_items())))
+        ]
     else:
         return list(cast(Iterable[ResultOrResultList], make_items()))
 
@@ -564,11 +573,15 @@ def for_each(
     if len(results) == 1 and isinstance(results[0].resolved, MutableSequence):
         result = _for_each(foreach, enumerate(results[0].resolved), ctx)  # hack!
     else:
-        result = _for_each(foreach, enumerate(r.external or r.resolved for r in results), ctx)
+        result = _for_each(
+            foreach, enumerate(r.external or r.resolved for r in results), ctx
+        )
     return ResultsList(result, ctx)  # ResultsList[List[ResultOrResultList]]]
 
 
-def for_each_func(foreach: Union[Mapping, str], ctx: RefContext) -> List[ResultOrResultList]:
+def for_each_func(
+    foreach: Union[Mapping, str], ctx: RefContext
+) -> List[ResultOrResultList]:
     results = ctx.currentResource
     if isinstance(results, Mapping):
         return _for_each(foreach, results.items(), ctx)
@@ -657,6 +670,7 @@ def eval_ref(
             return results
     return _eval_ref_results(val, ctx)
 
+
 def _eval_ref_results(val, ctx):
     mappedVal = Results._map_value(val, ctx)
     if isinstance(mappedVal, Result):
