@@ -1,6 +1,7 @@
 # Copyright (c) 2020 Adam Souzis
 # SPDX-License-Identifier: MIT
 from collections.abc import Mapping
+import datetime
 import os.path
 import hashlib
 import json
@@ -24,7 +25,7 @@ from .runtime import (
     CapabilityInstance,
     RelationshipInstance,
     ArtifactInstance,
-    TopologyInstance
+    TopologyInstance,
 )
 from .util import UnfurlError, to_enum, sensitive_str, get_base_dir, taketwo
 from .repo import split_git_url, RepoView, GitRepo
@@ -46,8 +47,7 @@ _basepath = os.path.abspath(os.path.dirname(__file__))
 
 
 def relabel_dict(environment: Dict, localEnv: "LocalEnv", key: str) -> Dict:
-    """Retrieve environment dictionary and remap any values that are strings in the dictionary by treating them as keys into an environment.
-    """
+    """Retrieve environment dictionary and remap any values that are strings in the dictionary by treating them as keys into an environment."""
     connections = environment.get(key)
     if not connections:
         return {}
@@ -87,7 +87,7 @@ class Manifest(AttributeManager):
 
     rootResource: Optional[TopologyInstance] = None
 
-    def __init__(self, path: Optional[str], localEnv: Optional["LocalEnv"]=None):
+    def __init__(self, path: Optional[str], localEnv: Optional["LocalEnv"] = None):
         super().__init__(yaml)
         self.localEnv = localEnv
         self.path = path
@@ -106,6 +106,7 @@ class Manifest(AttributeManager):
         self.imports = Imports()
         self.imports.manifest = self
         self._importedManifests: Dict = {}
+        self.cache: Dict[str, Any] = {}
 
     def _add_repositories_from_environment(self):
         assert self.localEnv
@@ -114,11 +115,14 @@ class Manifest(AttributeManager):
         for key, value in relabel_dict(context, self.localEnv, "repositories").items():
             if "." in key:  # assume it's a package not a repository name
                 assert isinstance(value, dict)
-                self.package_specs[key] = PackageSpec(key, value.get("url"), value.get("revision"))
+                self.package_specs[key] = PackageSpec(
+                    key, value.get("url"), value.get("revision")
+                )
             else:
                 repositories[key] = value
         env_package_spec = context.get("variables", {}).get(
-            "UNFURL_PACKAGE_RULES", os.getenv("UNFURL_PACKAGE_RULES"))
+            "UNFURL_PACKAGE_RULES", os.getenv("UNFURL_PACKAGE_RULES")
+        )
         if env_package_spec:
             for key, value in taketwo(env_package_spec.split()):
                 self.package_specs[key] = PackageSpec(key, value, None)
@@ -207,6 +211,10 @@ class Manifest(AttributeManager):
     def get_base_dir(self):
         return "."
 
+    @property
+    def deployment(self) -> str:
+        return os.path.basename(os.path.dirname(self.path)) if self.path else ""
+
     @property
     def loader(self) -> Optional["AnsibleLoader"]:
         return (
@@ -282,7 +290,9 @@ class Manifest(AttributeManager):
         configChange.previousId = changeSet.get("previousId")
         configChange.target = changeSet.get("target", "")
         assert isinstance(configChange.target, str)
-        configChange.operation = changeSet.get("implementation", {}).get("operation", "")
+        configChange.operation = changeSet.get("implementation", {}).get(
+            "operation", ""
+        )
         assert isinstance(configChange.operation, str)
 
         configChange.inputs = changeSet.get("inputs")  # type: ignore
@@ -338,13 +348,14 @@ class Manifest(AttributeManager):
     #         postConditions=spec.get("postConditions"),
     #     )
 
-    def _create_requirement(self, key, val) -> RelationshipInstance:
+    def _create_requirement(self, key, val) -> Optional[RelationshipInstance]:
         # parent will be the capability, should have already been created
         capabilityId = val.get("capability")
         if not capabilityId:
             nodeId = val.get("node")
             if not nodeId:
-                raise UnfurlError(f"requirement is missing capability {key}")
+                logger.warning(f"skipping requirement {key}: no node or capability specified")
+                return None
         capability = capabilityId and self.get_root_resource().query(capabilityId)
         if not capability or not isinstance(capability, CapabilityInstance):
             raise UnfurlError(f"can not find capability {capabilityId}")
@@ -371,9 +382,13 @@ class Manifest(AttributeManager):
             for req in resourceSpec["requirements"]:
                 key, val = next(iter(req.items()))
                 requirement = self._create_requirement(key, val)
+                if not requirement:
+                    continue
                 requirement._source = resource
                 # XXX investigate and re-enable this assert
-                assert requirement in resource.requirements, f"{requirement} not in {resource.requirements} for {rname}"
+                assert (
+                    requirement in resource.requirements
+                ), f"{requirement} not in {resource.requirements} for {rname}"
 
         if resourceSpec.get("artifacts"):
             for key, val in resourceSpec["artifacts"].items():
@@ -487,7 +502,9 @@ class Manifest(AttributeManager):
         )
         return repo, filePath, revision, bare
 
-    def add_repository(self, repo: Optional[GitRepo], toscaRepository: Repository, file_name: str) -> RepoView:
+    def add_repository(
+        self, repo: Optional[GitRepo], toscaRepository: Repository, file_name: str
+    ) -> RepoView:
         # add or replace the repository
         repository: Optional[RepoView] = self.repositories.get(toscaRepository.name)
         if repository:
@@ -499,13 +516,13 @@ class Manifest(AttributeManager):
                 raise UnfurlError(
                     f'Repository "{toscaRepository.name}" already defined'
                 )
-        repository = RepoView(
-            toscaRepository, repo, file_name
-        )
+        repository = RepoView(toscaRepository, repo, file_name)
         self.repositories[toscaRepository.name] = repository
         return repository
 
-    def _update_repositories(self, config, inlineRepositories=None, resolver: Optional[ImportResolver] = None) -> Dict[str, dict]:
+    def _update_repositories(
+        self, config, inlineRepositories=None, resolver: Optional[ImportResolver] = None
+    ) -> Dict[str, dict]:
         # Called during parse time when including files
         if not resolver:
             resolver = self.get_import_resolver(self)
@@ -609,7 +626,7 @@ class Manifest(AttributeManager):
                 # replace spec with just its name
                 artifactTpl["repository"] = reponame
                 assert repo.get("url"), f"bad inline repository definition: {repo}"
-                inlineRepository = {reponame: dict(url=repo['url'])}
+                inlineRepository = {reponame: dict(url=repo["url"])}
         else:
             artifactTpl = dict(file=templatePath)
 
@@ -631,18 +648,19 @@ class Manifest(AttributeManager):
                 return "", None
             raise UnfurlError(msg)
 
-        tpl = CommentedMap()
         resolver = self.get_import_resolver(warnWhenNotFound, config=expanded)
-        tpl["repositories"] = self._update_repositories(
+        repositories = self._update_repositories(
             expanded or yamlConfig.config, inlineRepository, resolver
         )
         loader = toscaparser.imports.ImportsLoader(
             None,
             get_base_dir(baseDir),
-            tpl=tpl,
+            repositories=repositories,
             resolver=resolver,
         )
-        import_spec = dict(file=artifactTpl["file"], repository=artifactTpl.get("repository"))
+        import_spec = dict(
+            file=artifactTpl["file"], repository=artifactTpl.get("repository")
+        )
         path, doc = loader._load_import_template(None, import_spec)
         if doc is None:
             logger.warning(
@@ -650,23 +668,28 @@ class Manifest(AttributeManager):
             )
         return path, doc
 
-    def get_import_resolver(self, ignoreFileNotFound=False, expand=False, config=None) -> ImportResolver:
+    def get_import_resolver(
+        self, ignoreFileNotFound=False, expand=False, config=None
+    ) -> ImportResolver:
         return ImportResolver(self, ignoreFileNotFound, expand, config)
 
-    def last_commit_time(self) -> float:
+    def last_commit_time(self) -> Optional[datetime.datetime]:
         # return seconds (0 if not found)
         repo = self.repo
         if not repo:
-            return 0
+            return None
         try:
             # find the revision that last modified this file before or equal to the current revision
             # (use current revision to handle branches)
-            commits = list(repo.repo.iter_commits(repo.revision, self.path or "", max_count=1))
+            commits = list(
+                repo.repo.iter_commits(repo.revision, self.path or "", max_count=1)
+            )
         except ValueError:
-            return 0
+            return None
         if commits:
-            return commits[0].committed_date
-        return 0
+            return commits[0].committed_datetime
+        return None
+
 
 # unused
 # class SnapShotManifest(Manifest):
