@@ -17,6 +17,7 @@ from .util import UnfurlError, save_to_file
 from toscaparser.repositories import Repository
 from ruamel.yaml.comments import CommentedMap
 import logging
+
 if TYPE_CHECKING:
     from .packages import Package
 
@@ -66,14 +67,17 @@ def normalize_git_url(url: str, hard: int = 0):
     return url
 
 
-def sanitize_url(url: str) -> str:
+def sanitize_url(url: str, redact=True) -> str:
     if "://" in url and "@" in url:  # sanitize
         parts = urlparse(url)
         # XXXX out user and password
         user, sep, host = parts.netloc.rpartition("@")
         if user:
-            user, sep, password = user.partition(':')
-            netloc = f"XXXXX{':XXXXX' if password else ''}@{host}"
+            user, sep, password = user.partition(":")
+            if redact:
+                netloc = f"XXXXX{':XXXXX' if password else ''}@{host}"
+            else:
+                netloc = host
             return parts._replace(netloc=netloc).geturl()
     return url
 
@@ -120,23 +124,25 @@ def split_git_url(url):
     return url, "", ""
 
 
-# git fetch <remote> 'refs/tags/*:refs/tags/*' if our clones are shallow 
+# git fetch <remote> 'refs/tags/*:refs/tags/*' if our clones are shallow
 @lru_cache(None)  # XXX won't get up dates in server mode
 def get_remote_tags(url, pattern="*") -> List[str]:
     # https://github.com/gitpython-developers/GitPython/issues/1071
     # https://myshittycode.com/2020/10/02/git-querying-tags-without-cloning-the-repository/
     # -v:refname is version sort in reverse order
     # -c versionsort.suffix=- ensures 1.0.0-XXXXXX comes before 1.0.0.
-    blob = git.cmd.Git()(c="versionsort.suffix=-").ls_remote(url, pattern, sort='-v:refname', tags=True)
+    blob = git.cmd.Git()(c="versionsort.suffix=-").ls_remote(
+        url, pattern, sort="-v:refname", tags=True
+    )
     # len("b90df3d12413db22d051db1f7c7286cdd2f00b66\trefs/tags/") == 51
     # filter out ^{} references (see https://stackoverflow.com/questions/12938972/what-does-mean-in-git)
-    tags = [line[51:] for line in blob.split('\n') if not line.endswith("^{}")]
+    tags = [line[51:] for line in blob.split("\n") if not line.endswith("^{}")]
     logger.debug("got %s remote tags with pattern %s from %s", len(tags), pattern, url)
     return tags
 
 
 class _ProgressPrinter(git.RemoteProgress):
-    gitUrl = ''
+    gitUrl = ""
 
     def update(self, op_code, cur_count, max_count=None, message=""):
         # we use print instead of logging because we don't want to clutter logs with this message
@@ -161,7 +167,9 @@ class Repo(abc.ABC):
         return None
 
     @staticmethod
-    def find_git_working_dirs(rootDir, include_root, gitDir=".git") -> Dict[str, "RepoView"]:
+    def find_git_working_dirs(
+        rootDir, include_root, gitDir=".git"
+    ) -> Dict[str, "RepoView"]:
         working_dirs: Dict[str, "RepoView"] = {}
         for root, dirs, files in os.walk(rootDir):
             if Repo.update_git_working_dirs(working_dirs, root, dirs, gitDir):
@@ -170,7 +178,9 @@ class Repo(abc.ABC):
         return working_dirs
 
     @staticmethod
-    def update_git_working_dirs(working_dirs, root, dirs, gitDir=".git") -> Optional[str]:
+    def update_git_working_dirs(
+        working_dirs, root, dirs, gitDir=".git"
+    ) -> Optional[str]:
         if gitDir in dirs and is_git_worktree(root, gitDir):
             assert os.path.isdir(root), root
             repo = GitRepo(git.Repo(root))
@@ -242,10 +252,7 @@ class Repo(abc.ABC):
             # e.g. extract tosca-parser from https://github.com/onecommons/tosca-parser.git
             if name_only:
                 path = os.path.basename(path)
-            name = (
-                os.path.splitext(path)[0]
-                or parts.netloc
-            )
+            name = os.path.splitext(path)[0] or parts.netloc
         assert not name.endswith(".git"), name
         return name
 
@@ -319,7 +326,9 @@ def find_dirty_secrets(working_dir):
 class RepoView:
     # view of Repo optionally filtered by path
     # XXX and revision too
-    def __init__(self, repository: Union[dict, Repository], repo: Optional["GitRepo"], path="") -> None:
+    def __init__(
+        self, repository: Union[dict, Repository], repo: Optional["GitRepo"], path=""
+    ) -> None:
         if isinstance(repository, dict):
             # required keys: name, url
             tpl = repository.copy()
@@ -337,7 +346,9 @@ class RepoView:
         self.repo = repo
         self.path = path
         if repo and path and self.repository:
-            self.repository.url = repo.get_url_with_path(path, False, self.revision or "")
+            self.repository.url = repo.get_url_with_path(
+                path, False, self.revision or ""
+            )
         self.readOnly = not repo
 
     @property
@@ -493,27 +504,40 @@ class GitRepo(Repo):
     def __init__(self, gitrepo: git.Repo):
         self.repo = gitrepo
         self.url = self.working_dir or str(gitrepo.git_dir)
-        if gitrepo.remotes:
+        remote = self.remote
+        if remote:
             # note: these might not look like absolute urls, e.g. git@github.com:onecommons/unfurl.git
-            try:
-                remote = gitrepo.remotes["origin"]
-            except Exception:
-                remote = gitrepo.remotes[0]
             self.url = remote.url
+        self.push_url: Optional[str] = None
 
-    def add_transient_credentials(self, username, password):
-        assert "@" not in self.url, self.url
-        replacement = f'url."{username}:{password}@{self.url}".insteadOf "{self.url}"'
-        self.repo.git.set_persistent_git_options(c=replacement)
+    def add_transient_push_credentials(self, username, password):
+        if not self.remote:
+            return
+        url = add_user_to_url(self.url, username, password)
+        if self.push_url is None:
+            self.push_url = self.repo.git.remote("get-url", "--push", self.remote.name)
+        replacement = f'url."{url}".insteadOf="{self.push_url}"'
+        # _git_options get cleared after next git command is issued
+        self.repo.git._git_options = self.repo.git.transform_kwargs(
+            split_single_char_options=True, c=replacement
+        )
 
-    def set_url_credentials(self, username, password):
-        try:
-            remote = self.repo.remotes["origin"]
-        except Exception:
-            remote = self.repo.remotes[0]
-        new_url = add_user_to_url(remote.url, username, password)
-        # replace the url
-        remote.set_url(new_url, remote.url)
+    def set_url_credentials(
+        self, username: str, password: str, fetch_only=False
+    ) -> None:
+        remote = self.remote
+        if remote:
+            if username or password:
+                new_url = add_user_to_url(remote.url, username, password)
+            else:
+                # clear credentials
+                new_url = sanitize_url(remote.url, False)
+            remote.set_url(new_url, remote.url)
+            self.url = new_url
+            if fetch_only:
+                # exclude credentials from the push url
+                self.push_url = sanitize_url(remote.url, False)
+                remote.set_url(self.push_url, push=True)
 
     @property
     def working_dir(self) -> str:
@@ -532,6 +556,17 @@ class GitRepo(Repo):
             return ""
         return self.repo.head.commit.hexsha
 
+    @property
+    def remote(self) -> Optional[git.Remote]:
+        gitrepo = self.repo
+        if gitrepo.remotes:
+            # note: these might not look like absolute urls, e.g. git@github.com:onecommons/unfurl.git
+            try:
+                return gitrepo.remotes["origin"]
+            except Exception:
+                return gitrepo.remotes[0]
+        return None
+
     def resolve_rev_spec(self, revision):
         try:
             return self.repo.commit(revision).hexsha
@@ -593,14 +628,16 @@ class GitRepo(Repo):
         # note: sets cwd to working_dir
         return gitcmd.execute(  # type: ignore
             call, with_exceptions=with_exceptions, with_extended_output=True, **kw
-        )  
+        )
 
     def add_to_local_git_ignore(self, rule):
         path = os.path.join(self.repo.git_dir, "info")
         if not os.path.exists(path):
             os.makedirs(path)
-        with open(os.path.join(path, "exclude"), "a") as f:
+        exclude_path = os.path.join(path, "exclude")
+        with open(exclude_path, "a") as f:
             f.write("\n" + rule + "\n")
+        return exclude_path
 
     def show(self, path, commitId):
         if self.working_dir and os.path.isabs(path):
@@ -654,32 +691,41 @@ class GitRepo(Repo):
         # https://gitpython.readthedocs.io/en/stable/reference.html?highlight=is_dirty#git.repo.base.Repo.is_dirty
         return self.repo.is_dirty(untracked_files=untracked_files, path=path or None)
 
-    def pull(self, remote="origin", revision=None, ff_only=True, **kw):
+    def pull(
+        self, remote="origin", revision=None, ff_only=True, with_exceptions=False, **kw
+    ) -> bool:
         if remote in self.repo.remotes:
             cmd = ["pull", remote, revision or "HEAD"]
             if ff_only:
                 cmd.append("--ff-only")
-            code, out, err = self.run_cmd(cmd, **kw)
+            code, out, err = self.run_cmd(cmd, with_exceptions=with_exceptions, **kw)
             if code:
                 logger.info(
                     "attempt to pull latest from %s into %s failed: %s %s",
-                    self.url,
+                    sanitize_url(self.url, True),
                     self.working_dir,
                     out,
                     err,
                 )
+                return False
             else:
                 logger.verbose(
                     "pull latest from %s into %s: %s %s",
-                    self.url,
+                    sanitize_url(self.url, True),
                     self.working_dir,
                     out,
                     err,
                 )
-            return not code
+                return True
         else:
             return False
 
+    def push(self, url: Optional[str] = None) -> None:
+        if url:
+            self.run_cmd(["push", url], with_exceptions=True)
+        elif self.remote:
+            self.remote.push().raise_if_error()
+
     def clone(self, newPath):
         # note: repo.clone uses bare path, which breaks submodule path resolution
         cloned = git.Repo.clone_from(
