@@ -3,17 +3,32 @@
 """
 Internal classes supporting the runtime.
 """
+import base64
 import collections
 from collections.abc import MutableSequence, Mapping
+import hashlib
+import math
 import os
+from random import choice
 import sys
 import os.path
-import six
 import re
 import ast
 import time
-from typing import TYPE_CHECKING, Iterator, List, MutableMapping, Tuple, Union, cast, Dict, Optional, Any, NewType
-from typing_extensions import Protocol
+from typing import (
+    TYPE_CHECKING,
+    Iterator,
+    List,
+    MutableMapping,
+    Tuple,
+    Union,
+    cast,
+    Dict,
+    Optional,
+    Any,
+    NewType,
+)
+from typing_extensions import Protocol, NoReturn
 from enum import Enum
 from urllib.parse import urlsplit
 
@@ -55,7 +70,7 @@ import ansible.template
 from ansible.parsing.dataloader import DataLoader
 from ansible.utils import unsafe_proxy
 from ansible.utils.unsafe_proxy import wrap_var, AnsibleUnsafeText, AnsibleUnsafeBytes
-from jinja2.runtime import DebugUndefined, make_logging_undefined
+from jinja2.runtime import DebugUndefined
 from toscaparser.elements.portspectype import PortSpec
 
 import logging
@@ -81,7 +96,7 @@ class Status(int, Enum):
             Status.pending: "white",
             Status.absent: "yellow",
         }[self]
- 
+
 
 # see "3.4.1 Node States" p74
 class NodeState(int, Enum):
@@ -167,14 +182,12 @@ set_eval_func("python", eval_python)
 #    return id(obj) in _secrets
 
 set_eval_func(
-    "sensitive",
-    lambda arg, ctx: wrap_sensitive_value(map_value(arg, ctx)), safe=True
+    "sensitive", lambda arg, ctx: wrap_sensitive_value(map_value(arg, ctx)), safe=True
 )
 
 
 set_eval_func(
-    "portspec",
-    lambda arg, ctx: PortSpec.make(map_value(arg, ctx)), safe=True
+    "portspec", lambda arg, ctx: PortSpec.make(map_value(arg, ctx)), safe=True
 )
 
 
@@ -257,7 +270,7 @@ def is_template(val, ctx=None):
     if isinstance(val, (AnsibleUnsafeText, AnsibleUnsafeBytes)):
         # already evaluated in a template, don't evaluate again
         return False
-    return isinstance(val, six.string_types) and not not _clean_regex.search(val)
+    return isinstance(val, str) and not not _clean_regex.search(val)
 
 
 class _VarTrackerDict(dict):
@@ -314,7 +327,7 @@ def _sandboxed_template(value: str, ctx: SafeRefContext, _UnfurlUndefined):
 
 
 def apply_template(value: str, ctx: RefContext, overrides=None) -> Any:
-    if not isinstance(value, six.string_types):
+    if not isinstance(value, str):
         msg = f"Error rendering template: source must be a string, not {type(value)}"
         if ctx.strict:
             raise UnfurlError(msg)
@@ -330,6 +343,30 @@ def apply_template(value: str, ctx: RefContext, overrides=None) -> Any:
     class _UnfurlUndefined(DebugUndefined):
         __slots__ = ()
 
+        def _fail_with_undefined_error(  # type: ignore
+            self, *args: Any, **kwargs: Any
+        ) -> "NoReturn":
+            try:
+                super()._fail_with_undefined_error(*args, **kwargs)
+            except self._undefined_exception as e:
+                msg = "Template: %s" % self._undefined_message
+                logger.warning(msg)
+                if ctx.task:  # already logged, so don't log
+                    UnfurlTaskError(ctx.task, msg, False)
+                raise e
+
+        # copied from Undefined, we need to reset _fail_with_undefined_error
+        __add__ = __radd__ = __sub__ = __rsub__ = _fail_with_undefined_error
+        __mul__ = __rmul__ = __div__ = __rdiv__ = _fail_with_undefined_error
+        __truediv__ = __rtruediv__ = _fail_with_undefined_error
+        __floordiv__ = __rfloordiv__ = _fail_with_undefined_error
+        __mod__ = __rmod__ = _fail_with_undefined_error
+        __pos__ = __neg__ = _fail_with_undefined_error
+        __call__ = _fail_with_undefined_error
+        __lt__ = __le__ = __gt__ = __ge__ = _fail_with_undefined_error
+        __int__ = __float__ = __complex__ = _fail_with_undefined_error
+        __pow__ = __rpow__ = _fail_with_undefined_error
+
         def __getattr__(self, name):
             if name == "__UNSAFE__":
                 # see AnsibleUndefined
@@ -343,11 +380,10 @@ def apply_template(value: str, ctx: RefContext, overrides=None) -> Any:
 
         def _log_message(self) -> None:
             msg = "Template: %s" % self._undefined_message
-            # XXX override _undefined_message: if self._undefined_obj is a Results then add its ctx._lastResource to the msg
-            #     see https://github.com/pallets/jinja/blob/7d72eb7fefb7dce065193967f31f805180508448/src/jinja2/runtime.py#L824
+            # XXX? if self._undefined_obj is a Results then add its ctx._lastResource to the msg
             logger.warning(msg)
             if ctx.task:  # already logged, so don't log
-                ctx.task._errors.append(UnfurlTaskError(ctx.task, msg, False))
+                UnfurlTaskError(ctx.task, msg, False)
 
         # see LoggingUndefined:
         def __str__(self) -> str:
@@ -437,7 +473,7 @@ def apply_template(value: str, ctx: RefContext, overrides=None) -> Any:
                 logger.warning(value[2:100] + "... see debug log for full report")
                 logger.debug(value, exc_info=True)
                 if ctx.task:
-                    ctx.task._errors.append(UnfurlTaskError(ctx.task, msg))
+                    UnfurlTaskError(ctx.task, msg)
         else:
             if value != oldvalue:
                 ctx.trace("successfully processed template:", value)
@@ -463,6 +499,8 @@ def apply_template(value: str, ctx: RefContext, overrides=None) -> Any:
 
                 # wrap result as AnsibleUnsafe so it isn't evaluated again
                 return wrap_var(value)
+            else:
+                ctx.trace("no modification after processing template:", value)
     finally:
         ctx.referenced.stop()
         if overrides:
@@ -607,13 +645,13 @@ def get_env(args, ctx: RefContext) -> Union[str, None, Dict[str, str]]:
 
     If the value of its argument is empty (e.g. [] or null), return the entire dictionary.
     """
-    env = cast(Dict[str, str], ctx.environ)
+    env = ctx.environ
     if not args:
         return env
 
     if isinstance(args, list):
         name = args[0]
-        default: Union[str, None] = args[1] if len(args) > 1 else None
+        default: Optional[str] = args[1] if len(args) > 1 else None
     else:
         name = args
         default = None
@@ -621,7 +659,7 @@ def get_env(args, ctx: RefContext) -> Union[str, None, Dict[str, str]]:
     if name in env:
         return env[name]
     else:
-        default = cast(Union[str, None], map_value(default, ctx))
+        default = cast(Optional[str], map_value(default, ctx))
         return default
 
 
@@ -656,6 +694,7 @@ class _EnvMapper(dict):
     """Resolve environment variable name to instance properties via the root template's requirements.
     Pattern should match _generate_env_names in to_json.py and set_context_vars above.
     """
+
     ctx: Optional[RefContext] = None
 
     def copy(self):
@@ -708,53 +747,166 @@ def to_env(args, ctx: RefContext):
 set_eval_func("to_env", to_env)
 
 
-def to_label(arg, allowed=r"\w", max=63, case="any", replace="", start="a-zA-Z", start_prepend="x", **kw):
-    r"""Convert a string to q label with the given constraints. 
-        If a dictionary or list, all keys and string values are converted.
+def _digest(arg: str, case: str, digest: Optional[str] = None) -> str:
+    m = hashlib.sha1()  # use same digest function as git
+    m.update(arg.encode("utf-8"))
+    if digest:
+        m.update(digest.encode("utf-8"))
+    if case == "any":
+        digest = (
+            base64.urlsafe_b64encode(m.digest())
+            .decode()
+            .strip("=")
+            .replace("_", "")
+            .replace("-", "")
+        )
+    else:
+        digest = base64.b32encode(m.digest()).decode().strip("=")
+        if case == "lower":
+            return digest.lower()
+        elif case == "upper":
+            return digest.upper()
+    return digest
+
+
+def _mid_truncate(label: str, replace: str, trunc: int) -> str:
+    if len(label) > trunc:
+        replace_len = len(replace)
+        mid = (trunc - replace_len) / 2
+        if mid <= 4:
+            return label[:trunc]
+        # trunc is odd, take one more from the beginning
+        return label[: math.ceil(mid)] + replace + label[-math.floor(mid) :]
+    return label
+
+
+_label_defaults = dict(
+    allowed=r"\w",
+    max=63,
+    case="any",
+    replace="",
+    start="a-zA-Z",
+    start_prepend="x",
+    end=r"\w",
+    sep="",
+    digest=None,
+    digestlen=-1,
+)
+
+
+def to_label(arg, **kw):
+    r"""Convert a string to a label with the given constraints.
+        If a dictionary, all keys and string values are converted.
+        If list, to_label is applied to each item and concatenated using ``sep``
 
     Args:
         arg (str or dict or list): Convert to label
-        allowed (str, optional): Allowed characters. Regex character ranges and character classes. 
+        allowed (str, optional): Allowed characters. Regex character ranges and character classes.
                                Defaults to "\w"  (equivalent to [a-zA-Z0-9_])
         replace (str, optional): String Invalidate. Defaults to "" (remove the characters).
-        start (str, optional): Allowed characters for the first character. Regex character ranges and character classes. 
+        start (str, optional): Allowed characters for the first character. Regex character ranges and character classes.
                                Defaults to "a-zA-Z"
         start_prepend (str, optional): If the start character is invalid, prepend with this string (Default: "x")
+        end (str, optional): Allowed trailing characters. Regex character ranges and character classes.
+                            Invalid characters are stripped. Defaults to "\w"  (equivalent to [a-zA-Z0-9_])
         max (int, optional): max length of label. Defaults to 63 (the maximum for a DNS name).
         case (str, optional): "upper", "lower" or "any" (no conversion). Defaults to "any".
+        sep (str, optional): Separator to use when concatenating a list. Defaults to ""
+        digestlen (int, optional): If a label is truncated, the length of the digest to include in the label. 0 to disable.
+                                Default: 3 or 2 if max < 32
     """
+    case: str = kw.get("case", _label_defaults["case"])
+    sep: str = kw.get("sep", _label_defaults["sep"])
+    digest: Optional[str] = kw.pop("digest", _label_defaults["digest"])
+    replace: str = kw.get("replace", _label_defaults["replace"])
+    elide_chars = replace
+
     if isinstance(arg, Mapping):
+        # convert keys and string values of the mapping, filtering out nulls
+        # only apply digest to values
         return {
-            to_label(n, allowed, max, case, replace): to_label(
-                v, allowed, max, case, replace
-            )
+            to_label(n, **kw): to_label(v, digest=digest, **kw)
             for n, v in arg.items()
             if v is not None
         }
-    elif isinstance(arg, list):
-        return [to_label(n, allowed, max, case, replace) for n in arg]
+
+    trunc: int = kw.pop("max", _label_defaults["max"])
+    assert trunc >= 1, trunc
+    checksum: int = kw.pop("digestlen", _label_defaults["digestlen"])
+
+    if checksum == -1:
+        checksum = 2 if trunc < 32 else 3
+    if checksum:
+        maxchecksum = min(trunc, checksum)
+    else:
+        maxchecksum = 0
+    if isinstance(arg, list):
+        if not arg:
+            return ""
+        # concatentate list
+        # adjust max for length of separators
+        trunc_chars = trunc - min(len(sep) * (len(arg) - 1), trunc - 1)
+        seg_max = max(trunc_chars // len(arg), 1)
+        segments = [str(n) for n in arg]
+        labels = [to_label(n, digestlen=0, max=9999, **kw) for n in segments]
+        length = sum(map(len, labels))
+        if length > trunc_chars or digest is not None:
+            # needs truncation and/or digest
+            # redistribute space from short segments
+            leftover = sum(map(lambda n: max(seg_max - len(n), 0), labels))
+            seg_max += leftover // len(labels)
+            labels = [_mid_truncate(seg, elide_chars, seg_max) for seg in labels]
+            if checksum:
+                # one of the labels was truncated, add a digest
+                trunc -= len(sep) + maxchecksum
+                hash = _digest("".join(segments), case, digest)[:maxchecksum]
+                if trunc <= 0:
+                    return hash
+                return sep.join(labels)[:trunc] + sep + hash
+        return sep.join(labels)[:trunc]
     elif isinstance(arg, str):
-        if arg and re.match(fr"[^{start}]", arg[0]):
+        start: str = kw.get("start", _label_defaults["start"])
+        start_prepend: str = kw.get("start_prepend", _label_defaults["start_prepend"])
+        allowed: str = kw.get("allowed", _label_defaults["allowed"])
+        end: str = kw.get("end", _label_defaults["end"])
+
+        if arg and re.match(rf"[^{start}]", arg[0]):
             val = start_prepend + arg
         else:
             val = arg
-        val = re.sub(fr"[^{allowed}]", replace, val)
+        if len(val) > trunc:
+            # heuristic: if we need to truncate, just remove the invalid characters
+            replace = ""
+        val = re.sub(rf"[^{allowed}]", replace, val)
+        while val and re.match(rf"[^{end}]", val[-1]):
+            val = val[:-1]
         if case == "lower":
             val = val.lower()
         elif case == "upper":
             val = val.upper()
-        return val[:max]
+
+        if maxchecksum and (digest is not None or len(val) > trunc):
+            trunc -= min(trunc, maxchecksum)
+            return (
+                _mid_truncate(val, elide_chars, trunc)
+                + _digest(arg, case, digest)[:maxchecksum]
+            )
+        else:
+            return _mid_truncate(val, elide_chars, trunc)
     else:
         return arg
 
 
 set_eval_func(
     "to_label",
-    lambda arg, ctx: to_label(map_value(arg, ctx), **ctx.kw), safe=True
+    lambda arg, ctx: to_label(map_value(arg, ctx), **map_value(ctx.kw, ctx)),
+    safe=True,
 )
 
 
-def to_dns_label(arg, max=63, **kw):
+def to_dns_label(
+    arg, allowed=r"\w-", start=r"\w", replace="--", case="lower", max=63, **kw
+):
     """
     Convert the given argument (see `to_label` for full description) to a DNS label (a label is the name separated by "." in a domain name).
     The maximum length of each label is 63 characters and can include
@@ -762,42 +914,51 @@ def to_dns_label(arg, max=63, **kw):
 
     Invalid characters are replaced with "--".
     """
-    return to_label(arg, allowed=r"\w-", start=r"\w", replace="--", max=max)
+    return to_label(
+        arg, allowed=allowed, start=start, replace=replace, case=case, max=max, **kw
+    )
 
 
 set_eval_func(
     "to_dns_label",
-    lambda arg, ctx: to_dns_label(map_value(arg, ctx), **ctx.kw), safe=True
+    lambda arg, ctx: to_dns_label(map_value(arg, ctx), **map_value(ctx.kw, ctx)),
+    safe=True,
 )
 
 
-def to_kubernetes_label(arg):
+def to_kubernetes_label(arg, allowed=r"\w_.-", replace="__", **kw):
     """
     See https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#syntax-and-character-set
 
     Invalid characters are replaced with "__".
     """
-    return to_label(arg, allowed=r"\w_.-", replace="__")
+    return to_label(arg, allowed=allowed, replace=replace, **kw)
 
 
 set_eval_func(
     "to_kubernetes_label",
-    lambda arg, ctx: to_kubernetes_label(map_value(arg, ctx)), safe=True
+    lambda arg, ctx: to_kubernetes_label(map_value(arg, ctx), **map_value(ctx.kw, ctx)),
+    safe=True,
 )
 
 
-def to_googlecloud_label(arg, max=63, **kw):
+def to_googlecloud_label(
+    arg, allowed=r"\w_-", case="lower", replace="__", max=63, **kw
+):
     """
     See https://cloud.google.com/resource-manager/docs/creating-managing-labels#requirements
 
     Invalid characters are replaced with "__".
     """
-    return to_label(arg, allowed=r"\w_-", case="lower", replace="__", max=max)
+    return to_label(arg, allowed=allowed, case=case, replace=replace, max=max, **kw)
 
 
 set_eval_func(
     "to_googlecloud_label",
-    lambda arg, ctx: to_googlecloud_label(map_value(arg, ctx), **ctx.kw), safe=True
+    lambda arg, ctx: to_googlecloud_label(
+        map_value(arg, ctx), **map_value(ctx.kw, ctx)
+    ),
+    safe=True,
 )
 
 
@@ -806,19 +967,21 @@ def get_ensemble_metadata(arg, ctx):
         return {}
     ensemble = ctx.task._manifest
     metadata = dict(
-        deployment=os.path.basename(os.path.dirname(ensemble.path)),
-        job=ctx.task.job.changeId
+        deployment=ensemble.deployment,
+        job=ctx.task.job.changeId,
     )
     if ensemble.repo:
         metadata["unfurlproject"] = ensemble.repo.project_path()
-        metadata["commit"] = ensemble.repo.revision[:8]
+        metadata["revision"] = ensemble.repo.revision[:8]
     environment = ensemble.localEnv and ensemble.localEnv.manifest_context_name
     if environment:
         metadata["environment"] = environment
     if arg:
         key = map_value(arg, ctx)
         if key == "project_namespace_subdomain" and ensemble.repo:
-            return ".".join(reversed(os.path.dirname(ensemble.repo.project_path()).split("/")))
+            return ".".join(
+                reversed(os.path.dirname(ensemble.repo.project_path()).split("/"))
+            )
         return metadata.get(key, "")
     else:
         return metadata
@@ -1267,7 +1430,10 @@ class DelegateAttributes:
                 return result
 
 
-AttributeChanges = NewType("AttributeChanges", Dict["InstanceKey", MutableMapping[str, Any]])
+AttributeChanges = NewType(
+    "AttributeChanges", Dict["InstanceKey", MutableMapping[str, Any]]
+)
+
 
 class ResourceChanges(collections.OrderedDict):
     """
@@ -1301,7 +1467,9 @@ class ResourceChanges(collections.OrderedDict):
     def sync(self, resource: "EntityInstance", changeId=None):
         """Update self to only include changes that are still live"""
         for k, v in list(self.items()):
-            current = cast(Optional["EntityInstance"], Ref(k).resolve_one(RefContext(resource)))
+            current = cast(
+                Optional["EntityInstance"], Ref(k).resolve_one(RefContext(resource))
+            )
             if current is not None:
                 attributes = v[self.attributesIndex]
                 if attributes:
@@ -1328,7 +1496,7 @@ class ResourceChanges(collections.OrderedDict):
 
     def add_statuses(self, changes):
         for name, change in changes.items():
-            assert not isinstance(change[1], six.string_types)
+            assert not isinstance(change[1], str)
             old = self.get(name)
             if old:
                 old[self.statusIndex] = change[1]
@@ -1339,7 +1507,9 @@ class ResourceChanges(collections.OrderedDict):
         for resource in resources:
             self["::" + resource["name"]] = [None, resource, None]
 
-    def update_changes(self, changes: AttributeChanges, statuses, resource, changeId=None):
+    def update_changes(
+        self, changes: AttributeChanges, statuses, resource, changeId=None
+    ):
         self.add_changes(changes)
         self.add_statuses(statuses)
         if resource:
@@ -1372,9 +1542,13 @@ class TopologyMap(dict):
         return len(tuple(self.resource.get_self_and_descendents()))
 
 
-LiveDependencies = NewType('LiveDependencies',
-                           Dict["InstanceKey", Tuple["EntityInstance",
-                                           Dict[str, Tuple[bool, Union[Result, Any]]]]])
+LiveDependencies = NewType(
+    "LiveDependencies",
+    Dict[
+        "InstanceKey",
+        Tuple["EntityInstance", Dict[str, Tuple[bool, Union[Result, Any]]]],
+    ],
+)
 
 
 class AttributeManager:
@@ -1389,6 +1563,7 @@ class AttributeManager:
     -- if a configurator wants to re-evaluate that attribute, it can create a dependency on it
     so to treat that as changed configuration.
     """
+
     validate = True
 
     # what about an attribute that is added to the spec that already exists in status?
@@ -1490,7 +1665,8 @@ class AttributeManager:
 
     def find_live_dependencies(self) -> Dict["InstanceKey", List["Dependency"]]:
         from .configurator import Dependency
-        dependencies: Dict[InstanceKey, List["Dependency"]] = {} 
+
+        dependencies: Dict[InstanceKey, List["Dependency"]] = {}
         for resource, attributes in self.attributes.values():
             overrides, specd = attributes._attributes.split()
             # items in overrides of type Result have been accessed during this transaction
@@ -1498,7 +1674,9 @@ class AttributeManager:
                 if isinstance(value, Result):
                     changed, isLive = self._check_attribute(specd, key, value, resource)
                     if isLive:
-                        dep = Dependency(resource.key + "::" + key, value, target=resource)
+                        dep = Dependency(
+                            resource.key + "::" + key, value, target=resource
+                        )
                         dependencies.setdefault(resource.key, []).append(dep)
         return dependencies
 
