@@ -17,7 +17,7 @@ import sys
 import itertools
 import json
 import datetime
-from typing import Any, Dict, List, Optional, Union, cast, TYPE_CHECKING
+from typing import Any, Dict, List, Optional, Tuple, Union, cast, TYPE_CHECKING
 from collections import Counter
 from collections.abc import Mapping, MutableSequence
 from urllib.parse import urlparse
@@ -33,6 +33,8 @@ from toscaparser.elements.scalarunit import get_scalarunit_class
 from toscaparser.elements.datatype import DataType
 from toscaparser.elements.portspectype import PortSpec
 from toscaparser.activities import ConditionClause
+from .yamlmanifest import YamlManifest
+from .merge import merge_dicts, patch_dict
 from .logs import sensitive, is_sensitive, getLogger
 from .tosca import is_function, get_nodefilters
 from .util import to_enum, UnfurlError
@@ -48,7 +50,7 @@ def _print(*args):
 
 
 def js_timestamp(ts: datetime.datetime) -> str:
-    jsts = ts.isoformat(' ', 'seconds')
+    jsts = ts.isoformat(" ", "seconds")
     if ts.utcoffset() is None:
         # if no timezone assume utc and append the missing offset
         return jsts + "+00:00"
@@ -148,7 +150,7 @@ VALUE_TYPES.update(
 def tosca_type_to_jsonschema(spec, propdefs, toscatype):
     jsonschema = dict(
         type="object",
-        properties = {p.name: tosca_schema_to_jsonschema(p, spec) for p in propdefs},
+        properties={p.name: tosca_schema_to_jsonschema(p, spec) for p in propdefs},
     )
     # add typeid:
     if toscatype:
@@ -209,7 +211,7 @@ def tosca_schema_to_jsonschema(p, spec):
         schema["title"] = toscaSchema.title or p.name
     if toscaSchema.default is not None and not is_value_computed(toscaSchema.default):
         schema["default"] = toscaSchema.default
-    elif toscaSchema.required:
+    if toscaSchema.required:
         schema["required"] = True
     if toscaSchema.description:
         schema["description"] = toscaSchema.description
@@ -228,10 +230,12 @@ def tosca_schema_to_jsonschema(p, spec):
             if valuetype_schema.constraints:  # merge constraints
                 constraints = valuetype_schema.constraints + constraints
         else:  # it's a complex type with properties:
-            propdefs = [p for p in dt.get_properties_def_objects() if is_property_user_visible(p)]
-            type_schema = tosca_type_to_jsonschema(
-                spec, propdefs, dt.type
-            )
+            propdefs = [
+                p
+                for p in dt.get_properties_def_objects()
+                if is_property_user_visible(p)
+            ]
+            type_schema = tosca_type_to_jsonschema(spec, propdefs, dt.type)
     if tosca_type not in ONE_TO_ONE_TYPES and "properties" not in schema:
         schema["$toscatype"] = tosca_type
     schema.update(type_schema)
@@ -251,8 +255,9 @@ def tosca_schema_to_jsonschema(p, spec):
 
 
 def _requirement_visibility(spec, name, req):
-    if name == "dependency":
-        return "omit"  # skip base TOSCA relationship type that every node has
+    if name in ["dependency", "installer"]:
+        # skip artifact requirements and the base TOSCA relationship type that every node has
+        return "omit"
     node = req.get("node")
     metadata = req.get("metadata") or {}
     if metadata.get("visibility"):
@@ -272,7 +277,7 @@ def _requirement_visibility(spec, name, req):
     return "inherit"
 
 
-def _get_req(req_dict):
+def _get_req(req_dict) -> Tuple[str, dict]:
     name, req = list(req_dict.items())[0]
     if isinstance(req, str):
         req = dict(node=req)
@@ -285,6 +290,58 @@ def expand_prefix(spec, nodetype: str):
     return nodetype.replace("tosca:", "tosca.nodes.")  # XXX
 
 
+def _find_req_typename(types, typename, reqname) -> str:
+    # have types[typename] go first
+    for target_reqs in types[typename]["requirements"]:
+        if target_reqs["name"] == reqname:
+            return target_reqs["resourceType"]
+
+    # XXX not very efficient
+    for t in types.values():
+        # mark_leaf_types() sets implementations, assume its has already been called
+        if "requirements" in t and not t["implementations"]:
+            continue  # not a leaf type
+        if typename in t["extends"]:
+            # type is typename or derived from typename
+            for target_reqs in t["requirements"]:
+                if target_reqs["name"] == reqname:
+                    return target_reqs["resourceType"]
+    return ""  # not found
+
+
+def _make_req(req_dict: dict, types=None, typename=None) -> Tuple[str, dict, dict]:
+    name, req = _get_req(req_dict)
+    reqobj: Dict[str, Any] = dict(
+        name=name,
+        description=req.get("description") or "",
+        __typename="RequirementConstraint",
+    )
+    metadata = req.get("metadata")
+
+    if metadata:
+        if "badge" in metadata:
+            reqobj["badge"] = metadata["badge"]
+        if "title" in metadata:
+            reqobj["title"] = metadata["title"]
+        if "icon" in metadata:
+            reqobj["icon"] = metadata["icon"]
+
+    if "occurrences" in req:
+        reqobj["min"] = req["occurrences"][0]
+        reqobj["max"] = req["occurrences"][1]
+
+    if req.get("node_filter"):
+        reqobj["node_filter"] = req["node_filter"]
+        if types is not None:
+            assert typename is not None
+            # we're called from annotate_requirements annotating a nested requirements constraint
+            reqtypename = _find_req_typename(types, typename, name)
+            if reqtypename:
+                _annotate_requirement(reqobj, reqtypename, types)
+
+    return name, req, reqobj
+
+
 def requirement_to_graphql(spec, req_dict):
     """
     type RequirementConstraint {
@@ -298,32 +355,22 @@ def requirement_to_graphql(spec, req_dict):
         badge: String
         visibility: String
         icon: String
+        inputsSchema: JSON
+        requirementsFilter: [RequirementConstraint!]
     }
     """
-    name, req = _get_req(req_dict)
+    name, req, reqobj = _make_req(req_dict)
+    if "min" not in reqobj:
+        # set defaults
+        reqobj["min"] = 1
+        reqobj["max"] = 1
+
     visibility = _requirement_visibility(spec, name, req)
     if visibility == "omit":
         return None
 
-    reqobj = dict(name=name, title=name, description=req.get("description") or "")
     if visibility != "inherit":
         reqobj["visibility"] = visibility
-    metadata = req.get("metadata")
-
-    if metadata:
-        if "badge" in metadata:
-            reqobj["badge"] = metadata["badge"]
-        if "title" in metadata:
-            reqobj["title"] = metadata["title"]
-        if "icon" in metadata:
-            reqobj["icon"] = metadata["icon"]
-
-    if "occurrences" in req:
-        reqobj["min"] = req["occurrences"][0]
-        reqobj["max"] = req["occurrences"][1]
-    else:
-        reqobj["min"] = 1
-        reqobj["max"] = 1
 
     if reqobj["max"] == 0:
         return None
@@ -339,11 +386,11 @@ def requirement_to_graphql(spec, req_dict):
     else:
         nodetype = req.get("capability")
         if not nodetype:
-            logger.warning("skipping constraint %s, there was no type specified ", req_dict)
+            logger.warning(
+                "skipping constraint %s, there was no type specified ", req_dict
+            )
             return None
     reqobj["resourceType"] = nodetype
-    if req.get("node_filter"):
-        reqobj["node_filter"] = req["node_filter"]
     return reqobj
 
 
@@ -614,7 +661,7 @@ def to_graphql_nodetypes(spec):
             node_type_to_graphql(spec, type_definition, types)
 
     mark_leaf_types(types)
-    annotate_properties(types)
+    annotate_requirements(types)
     return types
 
 
@@ -638,10 +685,10 @@ def add_capabilities_as_properties(schema, nodetype, spec):
     for cap in nodetype.get_capability_typedefs():
         if not cap.get_definition("properties"):
             continue
-        propdefs = [p for p in cap.get_properties_def_objects() if is_property_user_visible(p)]
-        schema[cap.name] = tosca_type_to_jsonschema(
-            spec, propdefs, cap.type
-        )
+        propdefs = [
+            p for p in cap.get_properties_def_objects() if is_property_user_visible(p)
+        ]
+        schema[cap.name] = tosca_type_to_jsonschema(spec, propdefs, cap.type)
 
 
 def add_capabilities_as_attributes(schema, nodetype, spec):
@@ -649,25 +696,10 @@ def add_capabilities_as_attributes(schema, nodetype, spec):
     for cap in nodetype.get_capability_typedefs():
         if not cap.get_definition("attributes"):
             continue
-        propdefs = [p for p in cap.get_attributes_def_objects() if is_property_user_visible(p)]
-        schema[cap.name] = tosca_type_to_jsonschema(
-            spec, propdefs, cap.type
-        )
-
-
-def _find_requirement_constraint(reqs, name):
-    for reqconstraint in reqs:
-        if reqconstraint["name"] == name:
-            return reqconstraint
-
-    # no requirement defined, use default
-    return dict(
-        name=name,
-        min=1,
-        max=1,
-        resourceType="tosca.nodes.Root",
-        __typename="RequirementConstraint",
-    )
+        propdefs = [
+            p for p in cap.get_attributes_def_objects() if is_property_user_visible(p)
+        ]
+        schema[cap.name] = tosca_type_to_jsonschema(spec, propdefs, cap.type)
 
 
 def _get_typedef(name: str, spec):
@@ -699,6 +731,30 @@ def template_properties_to_json(nodetemplate, visitor):
         yield dict(name=p.name, value=value)
 
 
+def _get_requirement(req, nodetemplate, spec, types):
+    name, req_dict = _get_req(req)
+    # we need to call _get_explicit_relationship() to make sure all the requirements relationships are created
+    # _get_explicit_relationship returns req_dict merged with its type definition
+    req_dict, rel_template = nodetemplate._get_explicit_relationship(name, req_dict)
+    if "constraint" in req_dict.get("metadata", {}):
+        # this happens when we import graphql json directly
+        reqconstraint = req_dict["metadata"]["constraint"]
+    else:
+        reqconstraint = requirement_to_graphql(spec, {name: req_dict})
+    if reqconstraint is None:
+        return None
+    _annotate_requirement(reqconstraint, reqconstraint["resourceType"], types)
+    reqjson = dict(
+        constraint=reqconstraint, name=name, match=None, __typename="Requirement"
+    )
+    if req_dict.get("node") and not _get_typedef(req_dict["node"], spec):
+        # it's not a type, assume it's a node template
+        # (the node template name might not match a template if it is only defined in the deployment blueprints)
+        match = req_dict["node"]
+        reqjson["match"] = match
+    return reqjson
+
+
 def nodetemplate_to_json(nodetemplate, spec, types, for_resource=False):
     """
     Returns json object as a ResourceTemplate:
@@ -722,7 +778,6 @@ def nodetemplate_to_json(nodetemplate, spec, types, for_resource=False):
     type Requirement {
       name: String!
       constraint: RequirementConstraint!
-      visibility: String
       match: ResourceTemplate
       target: Resource
     }
@@ -737,14 +792,15 @@ def nodetemplate_to_json(nodetemplate, spec, types, for_resource=False):
     json = dict(
         type=nodetemplate.type,
         name=nodetemplate.name,
-        title=nodetemplate.entity_tpl.get("metadata", {}).get("title") or nodetemplate.name,
+        title=nodetemplate.entity_tpl.get("metadata", {}).get("title")
+        or nodetemplate.name,
         description=nodetemplate.entity_tpl.get("description") or "",
         directives=nodetemplate.directives,
+        # __typename="ResourceTemplate",  # XXX
     )
     if "imported" in nodetemplate.entity_tpl:
         json["imported"] = nodetemplate.entity_tpl.get("imported")
 
-    jsonnodetype = types[nodetemplate.type]
     visitor = PropertyVisitor()
     json["properties"] = list(template_properties_to_json(nodetemplate, visitor))
     # if visitor.redacted and "predefined" not in nodetemplate.directives:
@@ -777,36 +833,10 @@ def nodetemplate_to_json(nodetemplate, spec, types, for_resource=False):
     has_visible_dependency = False
     ExceptionCollector.start()
     for req in nodetemplate.all_requirements:
-        name, req_dict = _get_req(req)
-        if name in ["dependency", "installer"]:
-            # skip artifact requirements and the base TOSCA relationship type that every node has
+        reqjson = _get_requirement(req, nodetemplate, spec, types)
+        if reqjson is None:
             continue
-        occurrences = req_dict.get("occurrences")
-        if occurrences and occurrences[1] == 0:
-            continue  # skip when max occurrences is 0
-
-        # we need to call _get_explicit_relationship() to make sure all the requirements relationships are created
-        reqDef, rel_template = nodetemplate._get_explicit_relationship(name, req_dict)
-        reqconstraint = _find_requirement_constraint(jsonnodetype["requirements"], name)
-        reqjson = dict(constraint=reqconstraint, name=name, __typename="Requirement")
-        req_metadata = req_dict.get("metadata") or req_dict.get("constraint") or {}
-        visibility = req_metadata and req_metadata.get("visibility")
-        if req_dict.get("node") and not _get_typedef(req_dict["node"], spec):
-            # it's not a type, assume it's a node template
-            # (the node template name might not match a template if it is only defined in the deployment blueprints)
-            match = req_dict["node"]
-            reqjson["match"] = match
-        if not visibility and reqjson.get("match"):
-            # user-defined templates should always have visibility set, so if it doesn't and the requirement is already set to a node
-            # assume it is internal and set to hidden.
-            if reqconstraint.get("visibility") != "visible" and not req_metadata.get(
-                "user_settable"
-            ):
-                # unless visibility wasn't explicitly set to visibile
-                visibility = "hidden"
-        if visibility:
-            reqjson["constraint"] = dict(reqjson["constraint"], visibility=visibility)
-        if visibility != "hidden":
+        if reqjson["constraint"].get("visibility") != "hidden":
             has_visible_dependency = True
         json["dependencies"].append(reqjson)
 
@@ -1082,8 +1112,9 @@ def _project_path(url):
     return pp
 
 
-def get_blueprint_from_topology(manifest, db):
+def get_blueprint_from_topology(manifest: YamlManifest, db):
     spec = manifest.tosca
+    assert spec
     # XXX cloud = spec.topology.primary_provider
     blueprint, root_name = to_graphql_blueprint(spec, db)
     templates = get_deployment_blueprints(manifest, blueprint, root_name, db)
@@ -1097,7 +1128,7 @@ def get_blueprint_from_topology(manifest, db):
     # the deployment template created for this deployment will have a "source" key
     # so if it doesn't (or one wasn't set) create a new one and set the current one as its "source"
     if "source" not in template:
-        title = os.path.basename(os.path.dirname(manifest.path))
+        title = os.path.basename(os.path.dirname(manifest.path)) if manifest.path else 'untitled'
         slug = slugify(title)
         template.update(
             dict(
@@ -1121,7 +1152,7 @@ def get_blueprint_from_topology(manifest, db):
     template["environmentVariableNames"] = list(_generate_env_names(spec, root_name))
     last_commit_time = manifest.last_commit_time()
     if last_commit_time:
-        template["commitTime"] = js_timestamp(datetime.datetime.fromtimestamp(last_commit_time))
+        template["commitTime"] = js_timestamp(last_commit_time)
     return blueprint, template
 
 
@@ -1224,8 +1255,12 @@ def add_graphql_deployment(manifest, db, dtemplate):
             if workflow == "undeploy" and deployment["status"] == Status.ok:
                 deployment["status"] = Status.absent
         deployment["summary"] = manifest.lastJob.get("summary")
-        deployment["deployTime"] = js_timestamp(datetime.datetime.strptime(
-            manifest.lastJob["startTime"], ChangeRecord.DateTimeFormat))
+        deployTimeString = manifest.lastJob.get(
+            "endTime", manifest.lastJob["startTime"]
+        )
+        deployment["deployTime"] = js_timestamp(
+            datetime.datetime.strptime(deployTimeString, ChangeRecord.DateTimeFormat)
+        )
 
     url = manifest.rootResource.attributes["outputs"].get("url")
     primary_resource = db["Resource"].get(primary_name)
@@ -1289,37 +1324,56 @@ def mark_leaf_types(types):
             jsontype["implementations"] = []
 
 
-def annotate_properties(types):
-    annotations = {}
+def annotate_requirements(types):
     for jsontype in types.values():
         for req in jsontype.get("requirements") or []:
-            if req.get("node_filter"):
-                annotations[req["resourceType"]] = req
-    for jsontype in types.values():
-        for typename in jsontype["extends"]:
-            node_filter = annotations.get(typename)
-            if node_filter:
-                map_nodefilter(node_filter, jsontype["inputsSchema"]["properties"])
+            _annotate_requirement(req, req["resourceType"], types)
 
 
-def map_nodefilter(filters, jsonprops):
-    ONE_TO_ONE_MAP = dict(object="map", array="list")
-    for name, value in get_nodefilters(filters, "properties"):
-        if name not in jsonprops or not isinstance(value, dict):
+def _annotate_requirement(req: dict, reqtypename: str, types: dict) -> None:
+    # req is a RequirementConstraint dict
+    if "node_filter" not in req:
+        return
+    node_filter = req["node_filter"]
+    reqtype = types[reqtypename]
+    req_filters = node_filter.get("requirements")
+    # node_filter properties might refer to properties that are only present on some subtypes
+    prop_filters: Dict = {}
+    for typename in reqtype["extends"]:
+        if typename not in types:
             continue
-        if "eval" in value:
+        subtype: Dict = types[typename]
+        inputsSchemaProperties = subtype["inputsSchema"]["properties"]
+        _map_nodefilter_properties(req, inputsSchemaProperties, prop_filters)
+
+    if prop_filters:
+        req["inputsSchema"] = dict(properties=prop_filters)
+    if req_filters:
+        req["requirementsFilter"] = [
+            _make_req(rf, types, reqtypename)[2] for rf in req_filters
+        ]
+    req.pop("node_filter")
+
+
+def _map_nodefilter_properties(filters, inputsSchemaProperties, jsonprops) -> dict:
+    ONE_TO_ONE_MAP = dict(object="map", array="list")
+    # {i["name"]: inputsSchemaProperties
+    for name, value in get_nodefilters(filters, "properties"):
+        if not isinstance(value, dict) or "eval" in value:
             # the filter declared an expression to set the property's value
-            # delete the annotated property from the target to it hide from the user
+            # mark the property to be deleted from the target inputschema so the user can't set it
             # (since they can't shouldn't set this property now)
-            del jsonprops[name]
-        else:
-            # update schema definition with the node filter's constraints
-            schema = jsonprops[name]
+            jsonprops[name] = None
+        elif name not in jsonprops and name in inputsSchemaProperties:
+            # check name in jsonprops so we only do this once
+            # use the inputs schema definition to determine the node filter's constraints
+            schema = inputsSchemaProperties[name]
             tosca_datatype = schema.get("$toscatype") or ONE_TO_ONE_MAP.get(
                 schema["type"], schema["type"]
             )
             constraints = ConditionClause(name, value, tosca_datatype).conditions
-            schema.update(map_constraints(schema["type"], constraints, schema))
+            jsonprops[name] = map_constraints(schema["type"], constraints, schema)
+    return jsonprops
 
 
 def _set_shared_instances(instances):
@@ -1376,13 +1430,17 @@ def to_environments(localEnv, existing=None):
     for ensemble_info in deployments.values():
         env_deployments[ensemble_info["environment"]] = ensemble_info["name"]
     defaults = localEnv.project.contexts.get("defaults")
-    default_imported_instances = _set_shared_instances(defaults and defaults.get("instances"))
+    default_imported_instances = _set_shared_instances(
+        defaults and defaults.get("instances")
+    )
     for name in localEnv.project.contexts:
         try:
             if name in env_deployments:
                 # we can reuse the localEnv if there's a distinct manifest that uses this environment
                 localEnv.manifest_context_name = name
-                localEnv.manifestPath = os.path.join(localEnv.project.projectRoot, env_deployments[name], "ensemble.yaml")
+                localEnv.manifestPath = os.path.join(
+                    localEnv.project.projectRoot, env_deployments[name], "ensemble.yaml"
+                )
                 localLocalEnv = localEnv
             else:
                 # this environment doesn't have any deployments so we have to create a new localEnv
@@ -1425,7 +1483,9 @@ def to_deployments(localEnv, existing=None):
     for manifest_path, dp in db["DeploymentPath"].items():
         try:
             # optimization: reuse localenv
-            localEnv.manifestPath = os.path.join(localEnv.project.projectRoot, manifest_path, "ensemble.yaml")
+            localEnv.manifestPath = os.path.join(
+                localEnv.project.projectRoot, manifest_path, "ensemble.yaml"
+            )
             environment = dp.get("environment") or "defaults"
             localEnv.manifest_context_name = environment
             deployments.append(to_deployment(localEnv))
@@ -1460,7 +1520,7 @@ def set_deploymentpaths(project, existing=None):
                 "project_id": ensemble_info.get("project_id"),
                 "pipelines": ensemble_info.get("pipelines", []),
                 "environment": ensemble_info["environment"],
-                "incremental_deploy": ensemble_info.get("incremental_deploy", False)
+                "incremental_deploy": ensemble_info.get("incremental_deploy", False),
             }
             if path in deployment_paths:
                 # merge duplicate entries
