@@ -102,7 +102,9 @@ def get_default_topology():
             tosca_definitions_version=TOSCA_VERSION,
             topology_template=dict(
                 node_templates={"_default": {"type": "tosca.nodes.Root"}},
-                relationship_templates={"_default": {"type": "tosca.relationships.Root"}},
+                relationship_templates={
+                    "_default": {"type": "tosca.relationships.Root"}
+                },
             ),
         )
         __default_topology = ToscaTemplate(yaml_dict_tpl=tpl)
@@ -166,12 +168,21 @@ class ToscaSpec:
         for nodespec in self.nodeTemplates.values():
             for req in nodespec.requirements.values():
                 for prop, value in req.get_nodefilter_properties():
-                    # annotate the target's properties
+                    # the target already has a node so treat the node filter
+                    # as coersive by annotating the target's properties
                     target = req.relationship and req.relationship.target
-                    if target and isinstance(value, dict) and "eval" in value:
-                        value.setdefault("vars", {})["SOURCE"] = dict(
-                            eval="::" + nodespec.name
-                        )
+                    if target:
+                        if isinstance(value, dict):
+                            if "eval" in value:
+                                value.setdefault("vars", {})["SOURCE"] = dict(
+                                    eval="::" + nodespec.name
+                                )
+                            elif "q" in value:
+                                value = value["q"]
+                            else:
+                                # we assume this is a constraint, so don't assign the value
+                                # XXX only continue if it actually looks like one
+                                continue
                         patch = dict(properties={prop: value})
                         _patch(target, patch, quote=True)
                         patched = True
@@ -180,10 +191,11 @@ class ToscaSpec:
                     target = req.relationship and req.relationship.target
                     if target:
                         matching_target_req = target.requirements.get(name)
-                        _patch(
-                            nodespec, value, tpl=matching_target_req.entity_tpl[name]
-                        )
-                        patched = True
+                        if matching_target_req:
+                            _patch(
+                                nodespec, value, tpl=matching_target_req.entity_tpl[name]
+                            )
+                            patched = True
 
         return patched
 
@@ -351,7 +363,7 @@ class ToscaSpec:
 
     def _get_project_dir(self, home=False):
         # hacky
-        if self.template.import_resolver:
+        if self.template and self.template.import_resolver:
             manifest = self.template.import_resolver.manifest
             if manifest.localEnv:
                 if home:
@@ -362,6 +374,7 @@ class ToscaSpec:
         return None
 
     def add_node_template(self, name, tpl, discovered=True):
+        assert self.template
         custom_types = None
         if "custom_types" in tpl:
             custom_types = tpl.pop("custom_types")
@@ -573,6 +586,7 @@ class ToscaSpec:
     def get_repository(self, name: str):
         return self.template and self.template.repositories.get(name)
 
+
 def find_env_vars(props_iter):
     for propdef, value in props_iter:
         datatype = propdef.entity
@@ -653,9 +667,9 @@ class EntitySpec(ResourceRef):
             )
 
         self.type = toscaNodeTemplate.type
-        self._isReferencedBy: Sequence[EntitySpec] = (
-            []
-        )  # this is referenced by another template or via property traversal
+        self._isReferencedBy: Sequence[
+            EntitySpec
+        ] = []  # this is referenced by another template or via property traversal
         # nodes have both properties and attributes
         # as do capability properties and relationships
         # but only property values are declared
@@ -663,8 +677,8 @@ class EntitySpec(ResourceRef):
         self.propertyDefs = toscaNodeTemplate.get_properties()
         self.attributeDefs = {}
         self.properties = CommentedMap(
-                [(prop.name, prop.value) for prop in self.propertyDefs.values()]
-            )
+            [(prop.name, prop.value) for prop in self.propertyDefs.values()]
+        )
         if toscaNodeTemplate.type_definition:
             # add attributes definitions
             attrDefs = toscaNodeTemplate.type_definition.get_attributes_def()
@@ -1054,9 +1068,9 @@ class RelationshipSpec(EntitySpec):
         # its connected through target, source, capability
         # its RelationshipType has valid_target_types
         if not template:
-            template = (
-                get_default_topology().topology_template.relationship_templates[0]
-            )
+            template = get_default_topology().topology_template.relationship_templates[
+                0
+            ]
             spec = ToscaSpec(get_default_topology())
         else:
             assert spec
@@ -1168,7 +1182,7 @@ def get_nodefilters(entity_tpl, key):
     if not isinstance(entity_tpl, dict):
         return
     nodefilter = entity_tpl.get("node_filter")
-    if nodefilter and key in nodefilter:
+    if nodefilter and nodefilter.get(key):
         for filter in nodefilter[key]:
             name, value = next(iter(filter.items()))
             yield name, value
@@ -1354,7 +1368,13 @@ class ArtifactSpec(EntitySpec):
         "file_extensions",
     )
 
-    def __init__(self, artifact_tpl, template: Optional[EntitySpec] = None, spec: Optional[ToscaSpec] = None, path=None):
+    def __init__(
+        self,
+        artifact_tpl,
+        template: Optional[EntitySpec] = None,
+        spec: Optional[ToscaSpec] = None,
+        path=None,
+    ):
         # 3.6.7 Artifact definition p. 84
         self.parentNode = template
         spec = template.spec if template else spec
@@ -1409,7 +1429,7 @@ class ArtifactSpec(EntitySpec):
             resolver = self.spec.template.import_resolver
 
         loader = toscaparser.imports.ImportsLoader(
-            None, self.base_dir, tpl=tpl, resolver=resolver
+            None, self.base_dir, repositories=tpl.get("repositories"), resolver=resolver
         )
         path, isFile, fragment = loader._resolve_import_template(
             None, self.as_import_spec()
