@@ -43,6 +43,7 @@ import io
 from contextlib import contextmanager
 import string
 import random
+from click.termui import unstyle
 
 try:
     from shutil import which
@@ -137,7 +138,6 @@ class UnfurlTaskError(UnfurlError):
         dependency=None,
     ):
         task = cast("ConfigTask", task)
-        message = f"{task.changeId} on {task.target.name} {task.name}: {message}"
         super().__init__(message, True, False)
         self.task = task
         task._errors.append(self)
@@ -457,6 +457,10 @@ def truncate_str(v: str) -> str:
     return v
 
 
+def clean_output(value: str) -> str:
+    return re.sub(r"[\x00-\x08\x0e-\x1f\x7f-\x9f]", "", unstyle(value))
+
+
 def get_random_password(count=12, prefix="uv", extra=None):
     srandom = random.SystemRandom()
     start = string.ascii_letters
@@ -557,7 +561,7 @@ class ChainMap(MutableMapping):
         self._maps = maps
 
     def copy(self):
-         # assume map implements copy()
+        # assume map implements copy()
         return ChainMap(*(m.copy() for m in self._maps))  # type: ignore
 
     def split(self) -> Tuple[MutableMapping, "ChainMap"]:
@@ -633,7 +637,7 @@ def substitute_env(contents, env=None, preserve_missing=False):
 
     def replace(m):
         if m.group(1):  # \ found
-            return m.group(0)[len(m.group(1)) - 1 or 1:]
+            return m.group(0)[len(m.group(1)) - 1 or 1 :]
         for name in m.group(2).split("|"):
             if name in env:
                 value = env[name]
