@@ -2,9 +2,8 @@
 # SPDX-License-Identifier: MIT
 """Loads and saves a ensemble manifest.
 """
-from __future__ import absolute_import
-from typing import Dict, List, Optional, Tuple
-import six
+import io
+from typing import Dict, List, Optional, Tuple, TYPE_CHECKING, cast
 import sys
 from collections.abc import MutableSequence, Mapping
 import numbers
@@ -30,6 +29,9 @@ from ruamel.yaml.comments import CommentedMap
 from codecs import open
 from ansible.parsing.dataloader import DataLoader
 
+if TYPE_CHECKING:
+    from .job import Job
+
 logger = getLogger("unfurl")
 
 _basepath = os.path.abspath(os.path.dirname(__file__))
@@ -177,7 +179,12 @@ class ReadOnlyManifest(Manifest):
     """Loads an ensemble from a manifest but doesn't instantiate the instance model."""
 
     def __init__(
-        self, manifest=None, path: Optional[str]=None, validate=True, localEnv=None, vault=None
+        self,
+        manifest=None,
+        path: Optional[str] = None,
+        validate=True,
+        localEnv=None,
+        vault=None,
     ):
         path = path or localEnv and localEnv.manifestPath
         if path:
@@ -190,7 +197,7 @@ class ReadOnlyManifest(Manifest):
             os.path.join(_basepath, "manifest-schema.json"),
             self.load_yaml_include,
             vault,
-            localEnv and localEnv.readonly
+            localEnv and localEnv.readonly,
         )
         if self.manifest.path:
             logger.debug("loaded ensemble manifest at %s", self.manifest.path)
@@ -252,9 +259,9 @@ class ReadOnlyManifest(Manifest):
         self.manifest.dump(out)
 
 
-def clone(localEnv, destPath):
+def clone(localEnv: LocalEnv, destPath) -> ReadOnlyManifest:
     clone = ReadOnlyManifest(localEnv=localEnv)
-    config = clone.manifest.config
+    config = cast(dict, clone.manifest.config)
     for key in ["status", "changes", "lastJob", "lock"]:
         config.pop(key, None)
     if "metadata" in config:
@@ -378,8 +385,8 @@ class YamlManifest(ReadOnlyManifest):
             self._load_resource_templates(resource_templates, node_templates, False)
         return True
 
-    def _configure_root(self, rootResource):
-        rootResource.imports = self.imports
+    def _configure_root(self, rootResource: TopologyInstance) -> None:
+        assert rootResource._templar
         if (
             self.manifest.vault and self.manifest.vault.secrets
         ):  # setBaseDir() may create a new templar
@@ -419,9 +426,7 @@ class YamlManifest(ReadOnlyManifest):
         rules = self.context.get("variables") or CommentedMap()
         for rel in root.requirements:
             rules.update(rel.merge_props(find_env_vars, True))
-        rules = serialize_value(
-            map_value(rules, root), resolveExternal=True
-        )
+        rules = serialize_value(map_value(rules, root), resolveExternal=True)
         root._environ = filter_env(rules, os.environ)
         paths = self.localEnv and self.localEnv.get_paths()
         if paths:
@@ -458,6 +463,7 @@ class YamlManifest(ReadOnlyManifest):
         for name, value in importsSpec.items():
             self.load_external_ensemble(name, value)
 
+        root.imports = self.imports
         return root
 
     def _load_resource_templates(self, templates, node_templates, virtual):
@@ -503,7 +509,9 @@ class YamlManifest(ReadOnlyManifest):
             )
 
         if "project" in location:
-            importedManifest = self.localEnv and self.localEnv.get_external_manifest(location)
+            importedManifest = self.localEnv and self.localEnv.get_external_manifest(
+                location
+            )
             if not importedManifest:
                 raise UnfurlError(
                     f"Can not import external ensemble '{name}': can't find project '{location['project']}'"
@@ -638,7 +646,8 @@ class YamlManifest(ReadOnlyManifest):
 
     def save_requirement(self, resource) -> Optional[Dict[str, Dict]]:
         if not resource.last_change and (
-            not resource.local_status or resource.local_status <= Status.ok
+            not resource.local_status
+            or resource.local_status <= Status.ok
             or resource.local_status == Status.pending
         ):
             # no reason to serialize requirements that haven't been instantiated
@@ -647,7 +656,9 @@ class YamlManifest(ReadOnlyManifest):
         status["capability"] = resource.parent.key
         return {name: status}
 
-    def _save_entity_if_instantiated(self, resource, checkstatus=True) -> Optional[Tuple[str, Dict]]:
+    def _save_entity_if_instantiated(
+        self, resource, checkstatus=True
+    ) -> Optional[Tuple[str, Dict]]:
         if "virtual" in resource.template.directives:
             return None
         if not resource.last_change and (
@@ -672,7 +683,11 @@ class YamlManifest(ReadOnlyManifest):
             return ret
         name, status = ret
 
-        if self.tosca and self.tosca.discovered and resource.template.name in self.tosca.discovered:
+        if (
+            self.tosca
+            and self.tosca.discovered
+            and resource.template.name in self.tosca.discovered
+        ):
             discovered[resource.template.name] = self.tosca.discovered[
                 resource.template.name
             ]
@@ -738,7 +753,7 @@ class YamlManifest(ReadOnlyManifest):
         )
         return status
 
-    def save_job_record(self, job):
+    def save_job_record(self, job: "Job"):
         """
         .. code-block:: YAML
 
@@ -754,15 +769,17 @@ class YamlManifest(ReadOnlyManifest):
         output = CommentedMap()
         output["changeId"] = job.changeId
         output["startTime"] = job.get_start_time()
+        output["endTime"] = job.get_end_time()
         if job.previousId:
             output["previousId"] = job.previousId
-        if job.masterJob:
+        if job.jobOptions.masterJob:
+            master_job = job.jobOptions.masterJob
             # if this was run by another ensemble's job
             masterJob = CommentedMap()
-            masterJob["path"] = job.masterJob.manifest.path
-            masterJob["changeId"] = job.masterJob.changeId
-            if job.masterJob.manifest.currentCommitId:
-                masterJob["startCommit"] = job.masterJob.manifest.currentCommitId
+            masterJob["path"] = master_job.manifest.path
+            masterJob["changeId"] = master_job.changeId
+            if master_job.manifest.currentCommitId:
+                masterJob["startCommit"] = master_job.manifest.currentCommitId
             output["masterJob"] = masterJob
         options = job.jobOptions.get_user_settings()
         output["workflow"] = options.pop("workflow", Defaults.workflow)
@@ -828,13 +845,13 @@ class YamlManifest(ReadOnlyManifest):
             job.out = self.manifest.save()
         return jobRecord, changes
 
-    def commit_job(self, job):
-        if job.planOnly:
+    def commit_job(self, job: "Job"):
+        if job.jobOptions.planOnly:
             return
         if job.dry_run:
             logger.info("printing results from dry run")
-            if not job.out and self.manifest.path:
-                job.out = sys.stdout
+            if not job.jobOptions.out and self.manifest.path:  # type: ignore
+                job.jobOptions.out = sys.stdout  # type: ignore
         jobRecord, changes = self.save_job(job)
         if not changes:
             logger.info("job run didn't make any changes; nothing to commit")
@@ -848,9 +865,9 @@ class YamlManifest(ReadOnlyManifest):
         if job.dry_run:
             return
 
-        if job.commit and self.repo:
-            if job.message is not None:
-                message = job.message
+        if job.jobOptions.commit and self.repo:
+            if job.jobOptions.message is not None:
+                message = job.jobOptions.message
             else:
                 message = self.get_default_commit_message()
 
@@ -858,6 +875,8 @@ class YamlManifest(ReadOnlyManifest):
             ensembleRepo = self.repositories["self"]
             if ensembleRepo.is_dirty():
                 ensembleRepo.commit(message, True)
+                if job.jobOptions.push and ensembleRepo.repo:
+                    ensembleRepo.repo.push()
 
     def get_default_commit_message(self):
         jobRecord = self.manifest.config.get("lastJob")
@@ -963,7 +982,7 @@ class YamlManifest(ReadOnlyManifest):
                 )
             changes = itertools.chain([jobRecord], newChanges)
             changelog["changes"] = list(changes)
-            output = six.StringIO()
+            output = io.StringIO()
             self.yaml.dump(changelog, output)
             if not os.path.isdir(os.path.dirname(fullPath)):
                 os.makedirs(os.path.dirname(fullPath))
